#!/usr/bin/env python3
"""
Market Monitor - Main coordinator for monitoring market events, orders, and positions.
"""

import logging
import asyncio
from datetime import datetime, timedelta, timezone
from typing import Optional, Dict, Any, List

from src.config.config import Config
from src.monitoring.alarm_manager import AlarmManager
from src.utils.token_display_formatter import get_formatter

# New processor/monitor/manager classes
from src.monitoring.order_fill_processor import OrderFillProcessor
from src.monitoring.position_synchronizer import PositionSynchronizer
from src.monitoring.external_event_monitor import ExternalEventMonitor
from src.monitoring.risk_cleanup_manager import RiskCleanupManager
from src.monitoring.drawdown_monitor import DrawdownMonitor

logger = logging.getLogger(__name__)

class MarketMonitorCache:
    """Simple data class to hold cached data for MarketMonitor and its delegates."""
    def __init__(self):
        self.cached_positions: List[Dict[str, Any]] = []
        self.cached_orders: List[Dict[str, Any]] = []
        self.cached_balance: Optional[Dict[str, Any]] = None
        self.last_cache_update: Optional[datetime] = None
        
        # last_known_orders is used by OrderFillProcessor to detect disappeared orders
        self.last_known_orders: set = set() 
        # last_known_positions is used by MarketMonitor._update_cached_data for logging count changes
        self.last_known_positions: Dict[str, Any] = {}

        # last_processed_trade_time_helper is used by OrderFillProcessor._check_for_recent_fills_for_order
        # This is distinct from ExternalEventMonitor.last_processed_trade_time which is for the main external fill processing
        self.last_processed_trade_time_helper: Optional[datetime] = None


class MarketMonitor:
    """Coordinates monitoring activities by delegating to specialized processors and managers."""
    
    def __init__(self, trading_engine, notification_manager=None):
        self.trading_engine = trading_engine
        self.notification_manager = notification_manager
        self._monitoring_active = False
        self._monitor_task = None
        
        self.alarm_manager = AlarmManager() # AlarmManager is standalone
        
        # Initialize the DrawdownMonitor
        stats = self.trading_engine.get_stats()
        self.drawdown_monitor = DrawdownMonitor(stats) if stats else None
        
        # Cache and Shared State
        self.cache = MarketMonitorCache()
        # Shared state for data that might be written by one manager and read by another
        # For now, primarily for external_stop_losses (written by RiskCleanupManager, read by ExternalEventMonitor)
        self.shared_state = {
            'external_stop_losses': {} # Managed by RiskCleanupManager
        }

        # Initialize specialized handlers
        self.order_fill_processor = OrderFillProcessor(
            trading_engine=self.trading_engine,
            notification_manager=self.notification_manager,
            market_monitor_cache=self.cache 
        )
        self.position_synchronizer = PositionSynchronizer(
            trading_engine=self.trading_engine,
            notification_manager=self.notification_manager,
            market_monitor_cache=self.cache
        )
        self.external_event_monitor = ExternalEventMonitor(
            trading_engine=self.trading_engine,
            notification_manager=self.notification_manager,
            alarm_manager=self.alarm_manager,
            market_monitor_cache=self.cache,
            shared_state=self.shared_state 
        )
        self.risk_cleanup_manager = RiskCleanupManager(
            trading_engine=self.trading_engine,
            notification_manager=self.notification_manager,
            market_monitor_cache=self.cache,
            shared_state=self.shared_state # Pass shared_state here
        )
        
        # Load minimal persistent state if any (most state is now in delegates or transient)
        self._load_state()
        
    async def start(self):
        if self._monitoring_active:
            logger.warning("Market monitor is already active")
            return
        
        self._monitoring_active = True
        logger.info("🔄 Market monitor started")
        
        await self._initialize_tracking()
        self._monitor_task = asyncio.create_task(self._monitor_loop())
    
    async def stop(self):
        if not self._monitoring_active:
            return
        
        self._monitoring_active = False
        if self._monitor_task:
            self._monitor_task.cancel()
            try:
                await self._monitor_task
            except asyncio.CancelledError:
                pass
        
        self._save_state() # Save minimal state if any
        logger.info("🛑 Market monitor stopped")
    
    def _load_state(self):
        """Load minimal MarketMonitor-specific state if necessary. Most state is now managed by delegates."""
        # Example: If AlarmManager needed to load state and was not self-sufficient
        # self.alarm_manager.load_alarms() 
        
        # last_processed_trade_time is loaded by ExternalEventMonitor itself.
        # external_stop_losses are in-memory in RiskCleanupManager for now.
        # last_processed_trade_time_helper for OrderFillProcessor can also be loaded if persisted.
        stats = self.trading_engine.get_stats()
        if not stats:
            logger.warning("⚠️ TradingStats not available, cannot load MarketMonitor helper states.")
            return
        try:
            helper_time_str = stats._get_metadata('order_fill_processor_last_processed_trade_time_helper')
            if helper_time_str:
                dt_obj = datetime.fromisoformat(helper_time_str)
                self.cache.last_processed_trade_time_helper = dt_obj.replace(tzinfo=timezone.utc) if dt_obj.tzinfo is None else dt_obj.astimezone(timezone.utc)
                logger.info(f"🔄 Loaded OrderFillProcessor helper state: last_processed_trade_time_helper = {self.cache.last_processed_trade_time_helper.isoformat()}")
        except Exception as e:
            logger.error(f"Error loading OrderFillProcessor helper state: {e}")

        logger.info("MarketMonitor _load_state: Minimal state loaded (most state handled by delegates).")

    def _save_state(self):
        """Save minimal MarketMonitor-specific state if necessary."""
        # self.alarm_manager.save_alarms()
        stats = self.trading_engine.get_stats()
        if not stats:
            logger.warning("⚠️ TradingStats not available, cannot save MarketMonitor helper states.")
            return
        try:
            if self.cache.last_processed_trade_time_helper:
                lptt_helper_utc = self.cache.last_processed_trade_time_helper.astimezone(timezone.utc)
                stats._set_metadata('order_fill_processor_last_processed_trade_time_helper', lptt_helper_utc.isoformat())
                logger.info(f"💾 Saved OrderFillProcessor helper state (last_processed_trade_time_helper) to DB: {lptt_helper_utc.isoformat()}")
        except Exception as e:
            logger.error(f"Error saving OrderFillProcessor helper state: {e}")
            
        logger.info("MarketMonitor _save_state: Minimal state saved.")
    
    async def _initialize_tracking(self):
        """Initialize basic tracking for cache."""
        try:
            orders = self.trading_engine.get_orders() or []
            # Initialize cache.last_known_orders for the first cycle of OrderFillProcessor
            self.cache.last_known_orders = {order.get('id') for order in orders if order.get('id')}
            logger.info(f"📋 Initialized cache with {len(orders)} open orders for first cycle comparison")

            positions = self.trading_engine.get_positions() or []
            # Initialize cache.last_known_positions for the first cycle of _update_cached_data logging
            self.cache.last_known_positions = {
                pos.get('symbol'): pos for pos in positions 
                if pos.get('symbol') and abs(float(pos.get('contracts', 0))) > 1e-9 # Use a small tolerance
            }
            logger.info(f"📊 Initialized cache with {len(positions)} positions for first cycle comparison")
            
            # Perform immediate startup sync using the PositionSynchronizer
            # This fetches fresh positions itself, so no need to pass `positions`
            await self.position_synchronizer._immediate_startup_auto_sync()
                    
        except Exception as e:
            logger.error(f"❌ Failed to initialize tracking: {e}", exc_info=True)
            self.cache.last_known_orders = set()
            self.cache.last_known_positions = {}
    
    async def _monitor_loop(self):
        try:
            loop_count = 0
            while self._monitoring_active:
                await self._update_cached_data() # Updates self.cache
                
                # Order of operations:
                # 1. Process fills and activate pending SLs based on fills (OrderFillProcessor)
                # 2. Check for external trades/events (ExternalEventMonitor)
                # 3. Check for various triggers and risk conditions (RiskCleanupManager)
                # 4. Periodic cleanup and sync tasks (RiskCleanupManager, PositionSynchronizer)

                await self.order_fill_processor._activate_pending_stop_losses_from_trades()
                await self.order_fill_processor._check_order_fills() 
                
                await self.external_event_monitor._check_price_alarms()
                await self.external_event_monitor._check_external_trades()
                
                await self.risk_cleanup_manager._check_pending_triggers()
                await self.risk_cleanup_manager._check_automatic_risk_management()
                await self.risk_cleanup_manager._check_external_stop_loss_orders()
                
                loop_count += 1
                if loop_count >= Config.MARKET_MONITOR_CLEANUP_INTERVAL_HEARTBEATS: # Use a config value
                    logger.info(f"Running periodic cleanup and sync tasks (Loop count: {loop_count})")
                    await self.risk_cleanup_manager._cleanup_orphaned_stop_losses()
                    await self.risk_cleanup_manager._cleanup_external_stop_loss_tracking()
                    await self.risk_cleanup_manager._cleanup_orphaned_pending_sl_activations() # User's requested new method
                    
                    await self.position_synchronizer._auto_sync_orphaned_positions()
                    
                    loop_count = 0
                
                await asyncio.sleep(Config.BOT_HEARTBEAT_SECONDS)
        except asyncio.CancelledError:
            logger.info("Market monitor loop cancelled")
            raise
        except Exception as e:
            logger.error(f"Error in market monitor loop: {e}", exc_info=True)
            if self._monitoring_active: # Attempt to restart loop after a delay
                logger.info("Attempting to restart market monitor loop after error...")
                await asyncio.sleep(Config.BOT_HEARTBEAT_SECONDS * 2) # Wait longer before restart
                # Re-create task to ensure clean start if previous one errored out badly.
                # This assumes _monitor_loop itself doesn't recurse on all exceptions.
                if self._monitoring_active: # Check again, stop() might have been called
                     self._monitor_task = asyncio.create_task(self._monitor_loop())

    async def _update_cached_data(self):
        """Continuously update cached exchange data for all components to use."""
        try:
            fresh_positions_list = self.trading_engine.get_positions() or []
            fresh_orders_list = self.trading_engine.get_orders() or []
            fresh_balance = self.trading_engine.get_balance()
            
            # Update cache object
            self.cache.cached_positions = fresh_positions_list
            self.cache.cached_orders = fresh_orders_list
            self.cache.cached_balance = fresh_balance
            self.cache.last_cache_update = datetime.now(timezone.utc)
            
            # Update drawdown monitor with the latest balance
            if self.drawdown_monitor and fresh_balance and fresh_balance.get('total') is not None:
                # fresh_balance['total'] is a dict like {'USDC': 1234.56, 'BTC': 0.001}
                usdc_balance = fresh_balance['total'].get('USDC', 0)
                if usdc_balance:
                    self.drawdown_monitor.update_balance(float(usdc_balance))

            logger.debug(f"🔄 Cache updated: {len(fresh_positions_list)} positions, {len(fresh_orders_list)} orders")

            current_exchange_position_map = {
                pos.get('symbol'): pos for pos in fresh_positions_list
                if pos.get('symbol') and abs(float(pos.get('contracts', 0))) > 1e-9
            }
            current_exchange_order_ids = {order.get('id') for order in fresh_orders_list if order.get('id')}

            if len(current_exchange_position_map) != len(self.cache.last_known_positions):
                logger.info(f"📊 Position count changed: {len(self.cache.last_known_positions)} → {len(current_exchange_position_map)}")
            
            if len(current_exchange_order_ids) != len(self.cache.last_known_orders):
                logger.info(f"📋 Order count changed: {len(self.cache.last_known_orders)} → {len(current_exchange_order_ids)}")

            # Update last_known_xxx in the cache for the *next* cycle's comparison
            # OrderFillProcessor uses last_known_orders for its own logic.
            # MarketMonitor uses last_known_positions for logging changes here.
            self.cache.last_known_positions = current_exchange_position_map
            # self.cache.last_known_orders is updated by OrderFillProcessor after it processes fills.
            # For the first run of _check_order_fills, it needs the initial set from _initialize_tracking.
            # Subsequent runs, it will use the state from its previous run.
            # So, MarketMonitor should not overwrite self.cache.last_known_orders here after initialization.
            
            stats = self.trading_engine.get_stats()
            if stats and fresh_positions_list:
                for ex_pos in fresh_positions_list: # Changed pos_data to ex_pos to match original
                    symbol = ex_pos.get('symbol')
                    if not symbol: 
                        continue

                    # Attempt to get the lifecycle_id for this symbol
                    db_trade = stats.get_trade_by_symbol_and_status(symbol, status='position_opened')
                    if db_trade:
                        lifecycle_id = db_trade.get('trade_lifecycle_id')
                        if not lifecycle_id:
                            logger.debug(f"No lifecycle_id for open position {symbol} in _update_cached_data, skipping detailed stats update.")
                            continue # Skip if no lifecycle ID, though this should ideally not happen for an open position

                        try:
                            # Extract all relevant data from exchange position (ex_pos)
                            current_size_from_ex = ex_pos.get('contracts')
                            current_position_size = float(current_size_from_ex) if current_size_from_ex is not None else None

                            entry_price_from_ex = ex_pos.get('entryPrice') or ex_pos.get('entryPx')
                            entry_price = float(entry_price_from_ex) if entry_price_from_ex is not None else None

                            mark_price_from_ex = ex_pos.get('markPrice') or ex_pos.get('markPx')
                            mark_price = float(mark_price_from_ex) if mark_price_from_ex is not None else None

                            unrealized_pnl_from_ex = ex_pos.get('unrealizedPnl')
                            unrealized_pnl = float(unrealized_pnl_from_ex) if unrealized_pnl_from_ex is not None else None

                            liquidation_price_from_ex = ex_pos.get('liquidationPrice')
                            liquidation_price = float(liquidation_price_from_ex) if liquidation_price_from_ex is not None else None

                            margin_used_from_ex = ex_pos.get('marginUsed')
                            margin_used = float(margin_used_from_ex) if margin_used_from_ex is not None else None

                            leverage_from_ex = ex_pos.get('leverage')
                            leverage = float(leverage_from_ex) if leverage_from_ex is not None else None

                            position_value_from_ex = ex_pos.get('notional')
                            position_value = float(position_value_from_ex) if position_value_from_ex is not None else None
                            
                            if position_value is None and mark_price is not None and current_position_size is not None:
                                position_value = abs(current_position_size) * mark_price

                            roe_from_ex = ex_pos.get('percentage')
                            unrealized_pnl_percentage_val = float(roe_from_ex) if roe_from_ex is not None else None

                            stats.update_trade_market_data(
                                trade_lifecycle_id=lifecycle_id, 
                                unrealized_pnl=unrealized_pnl, 
                                mark_price=mark_price,
                                current_position_size=current_position_size,
                                entry_price=entry_price, 
                                liquidation_price=liquidation_price,
                                margin_used=margin_used,
                                leverage=leverage,
                                position_value=position_value,
                                unrealized_pnl_percentage=unrealized_pnl_percentage_val
                            )
                        except (ValueError, TypeError) as e:
                            logger.warning(f"Could not parse full market data for {symbol} (Lifecycle: {lifecycle_id}) from {ex_pos}: {e}")
                    else:
                        # This case means there's a position on the exchange, but no 'position_opened' lifecycle in the DB.
                        # This should be handled by the _auto_sync_orphaned_positions logic in PositionSynchronizer.
                        # For _update_cached_data, we only update market data for KNOWN lifecycles.
                        logger.debug(f"No 'position_opened' lifecycle found for symbol {symbol} during _update_cached_data. Orphan sync should handle it.")
            
        except Exception as e:
            logger.error(f"❌ Error updating cached data: {e}", exc_info=True)

    # --- Cache Accessor Methods ---
    def get_cached_positions(self) -> List[Dict[str, Any]]:
        return self.cache.cached_positions

    def get_cached_orders(self) -> List[Dict[str, Any]]:
        return self.cache.cached_orders

    def get_cached_balance(self) -> Optional[Dict[str, Any]]:
        return self.cache.cached_balance

    def get_cache_age_seconds(self) -> Optional[float]:
        if self.cache.last_cache_update:
            return (datetime.now(timezone.utc) - self.cache.last_cache_update).total_seconds()
        return None

    # --- Alarm Management (delegated to AlarmManager, but MarketMonitor provides interface) ---
    def add_price_alarm(self, token: str, target_price: float, direction: str, user_id: int) -> Optional[int]:
        alarm_id = self.alarm_manager.add_alarm(token, target_price, direction, user_id)
        if alarm_id:
            logger.info(f"Price alarm added: ID {alarm_id} for {token} {direction} ${target_price}")
            # self._save_state() # AlarmManager should handle its own persistence if needed
        return alarm_id

    def remove_price_alarm(self, alarm_id: int, user_id: int) -> bool:
        removed = self.alarm_manager.remove_alarm(alarm_id, user_id)
        if removed:
            logger.info(f"Price alarm {alarm_id} removed by user {user_id}")
            # self._save_state()
        return removed

    def get_user_alarms(self, user_id: int) -> List[Dict[str, Any]]:
        return self.alarm_manager.get_alarms_by_user(user_id)

    def get_all_active_alarms(self) -> List[Dict[str, Any]]:
        return self.alarm_manager.get_all_active_alarms()

    # Methods that were moved are now removed from MarketMonitor.
    # _check_order_fills -> OrderFillProcessor
    # _process_disappeared_orders -> OrderFillProcessor
    # _activate_pending_stop_losses_from_trades -> OrderFillProcessor
    # _check_for_recent_fills_for_order -> OrderFillProcessor (helper)
    # _auto_sync_orphaned_positions -> PositionSynchronizer
    # _immediate_startup_auto_sync -> PositionSynchronizer
    # _estimate_entry_price_for_orphaned_position -> PositionSynchronizer (helper)
    # _send_startup_auto_sync_notification -> PositionSynchronizer (helper)
    # _check_external_trades -> ExternalEventMonitor
    # _check_price_alarms -> ExternalEventMonitor
    # _send_alarm_notification -> ExternalEventMonitor (helper)
    # _check_pending_triggers -> RiskCleanupManager
    # _check_automatic_risk_management -> RiskCleanupManager
    # _cleanup_orphaned_stop_losses -> RiskCleanupManager
    # _check_external_stop_loss_orders -> RiskCleanupManager
    # _cleanup_external_stop_loss_tracking -> RiskCleanupManager
    # _cleanup_orphaned_pending_sl_activations -> RiskCleanupManager (new stub)

    # Methods related to direct position/order processing like _process_filled_orders
    # and _update_position_tracking are implicitly part of OrderFillProcessor's logic now.
    # The complex internal logic of _check_external_trades for lifecycle updates is now within ExternalEventMonitor.
    # The state for `external_stop_losses` is now managed by `RiskCleanupManager` via `shared_state`.
    # The state for `last_processed_trade_time` for external fills is managed by `ExternalEventMonitor`.
    # The state for `last_processed_trade_time_helper` for `_check_for_recent_fills_for_order` is in `MarketMonitorCache`.